from typing import Optional

from locatieserver.client.utils import filter_defaults, http_get
from locatieserver.schema.lookup import LookupResponse

PATH = "lookup"


def lookup(
    id: str,
    rows: Optional[int] = 10,
    start: Optional[int] = 0,
    wt: Optional[str] = "json",
    indent: Optional[bool] = True,
    lat: Optional[float] = None,
    lon: Optional[float] = None,
    fq: Optional[str] = "type:(gemeente OR woonplaats OR weg OR postcode OR adres)",
) -> LookupResponse:
    """Lookup Service

    For more info: https://github.com/PDOK/locatieserver/wiki/API-Locatieserver#4lookup-service

    :param id: Bij de lookup-service moet de id-parameter worden gebruikt om het ID op te geven van het object
        waar naar gezocht dient te worden. De waarde van het ID kan worden bepaald door gebruik te maken van
        de suggest-service, of deze kan uit een extern systeem komen waar dit ID eerder in is opgeslagen.
    :param rows: Hiermee wordt opgegeven wat het maximale aantal rijen (ofwel resultaten)
        is dat teruggegeven moet worden op deze bevraging.
    :param start: Hiermee wordt opgegeven wat de index is van het eerste resultaat dat teruggegeven wordt.
        Dit is zero-based. In combinatie met de rows-parameter kunnen deze services gepagineerd worden bevraagd.
    :param wt: Hiermee wordt opgegeven wat het outputformaat is van de bevraging.
    :param indent: Hiermee kan worden opgegeven of de teruggegeven JSON ingesprongen (geïndenteerd) moet worden.
    :param lat: Werkt alleen in combinatie met `lon`.
        Hiermee kan een coördinaat (in lat/lon, ofwel WGS84) worden opgegeven.
        Met behulp van deze parameters worden de gevonden zoekresultaten gesorteerd op afstand van het meegegeven punt.
        Wanneer de locatie van de gebruiker bekend is, kan op deze manier effectiever worden gezocht.
        Het meegeven van een coördinaat is met name nuttig voor de suggest- en vrije geocoder-services.
        Hier worden meestal meerdere resultaten teruggegeven.
        Als decimaal scheidingsteken moet een punt worden opgegeven i.p.v. een komma.
    :param lon: Werkt alleen in combinatie met `lat`.
        Hiermee kan een coördinaat (in lat/lon, ofwel WGS84) worden opgegeven.
        Met behulp van deze parameters worden de gevonden zoekresultaten gesorteerd op afstand van het meegegeven punt.
        Wanneer de locatie van de gebruiker bekend is, kan op deze manier effectiever worden gezocht.
        Het meegeven van een coördinaat is met name nuttig voor de suggest- en vrije geocoder-services.
        Hier worden meestal meerdere resultaten teruggegeven.
        Als decimaal scheidingsteken moet een punt worden opgegeven i.p.v. een komma.
    :param fq: Hiermee kan een filter query worden opgegeven, bijv. `fq=bron:BAG`.
        Met `fq=*` kan de default filter query worden opgeheven.
    :return: LookupResponse schema
    """
    params = filter_defaults(
        lookup,
        id=id,
        rows=rows,
        start=start,
        wt=wt,
        indent=indent,
        lat=lat,
        lon=lon,
        fq=fq,
    )

    response = http_get(PATH, params)

    return LookupResponse.parse_raw(response.content)



# generated by datamodel-codegen:
#   filename:  error.json
#   timestamp: 2021-06-26T19:10:08+00:00

from __future__ import annotations

from typing import List

from locatieserver.schema.base import LocatieserverBaseModel


class Error(LocatieserverBaseModel):
    metadata: List[str]
    msg: str
    code: int


class ErrorResponse(LocatieserverBaseModel):
    error: Error



import inspect
from typing import Any, Callable, Dict

import httpx

from locatieserver.client.config import BASE_URL
from locatieserver.schema.error import ErrorResponse

DEFAULT_CACHE = {}


def get_defaults_from_function(func: Callable) -> Dict[str, Any]:
    if func.__name__ not in DEFAULT_CACHE:
        signature = inspect.signature(func)
        DEFAULT_CACHE[func.__name__] = {
            k: v.default for k, v in signature.parameters.items() if v.default is not inspect.Parameter.empty
        }

    return DEFAULT_CACHE[func.__name__]


def filter_defaults(func, **kwargs):

    defaults = get_defaults_from_function(func)

    non_default_values = {}

    for key, value in kwargs.items():
        if key not in defaults or value != defaults[key]:
            non_default_values[key] = value

    return non_default_values


class LocatieserverError(Exception):
    pass


class LocatieserverResponseError(LocatieserverError):
    pass


def http_get(path, params):
    response = httpx.get(BASE_URL + path, params=params)

    content_type = response.headers.get("Content-Type", None)

    if "json" not in content_type:
        raise LocatieserverError(f"Cannot handle content-types other then JSON, received {content_type}.")

    if response.status_code == 200:
        return response
    elif response.status_code == 400:
        error_response = ErrorResponse.parse_raw(response.content)

        raise LocatieserverResponseError(error_response.error.msg)

    return response



from locatieserver.schema.utils import Point, point_matcher


def test_point_regex():

    x = point_matcher.findall("POINT(5.10696041 52.06415055)")

    assert len(x[0]) == 2

    x = point_matcher.findall("POINT(179534 548410)")

    assert len(x[0]) == 2


def test_point():
    centroide_ll = Point("POINT(5.10696041 52.06415055)")

    assert centroide_ll.x == "5.10696041"
    assert centroide_ll.y == "52.06415055"

    centroide_ll = Point("POINT(10696041 06415055)")

    assert centroide_ll.x == "10696041"
    assert centroide_ll.y == "06415055"


def test_point_empty():
    centroide_ll = Point("")

    assert centroide_ll.x is None


def test_point_non_point_value():
    centroide_ll = Point("XYZ(1.0 2.0)")

    assert centroide_ll.x == "1.0"


def test_point_string_value():
    centroide_ll = Point("ABC")

    assert centroide_ll.x is None
